@@ -200,6 +200,8 @@ class OAuthRemoteApp(object):
                                  to forward to the request token url
                                  or authorize url depending on oauth
                                  version
+    :param request_token_method: the HTTP method that should be used for
+                                 the access_token_url. Default is ``GET``
     :param access_token_params: an optional dictionary of parameters to
                                 forward to the access token url
     :param access_token_method: the HTTP method that should be used for
@@ -222,6 +224,7 @@ class OAuthRemoteApp(object):
         consumer_key=None,
         consumer_secret=None,
         request_token_params=None,
+        request_token_method=None,
         access_token_params=None,
         access_token_method=None,
         content_type=None,
@@ -231,11 +234,6 @@ class OAuthRemoteApp(object):
         self.oauth = oauth
         self.name = name
 
-        if (not consumer_key or not consumer_secret) and not app_key:
-            raise TypeError(
-                'OAuthRemoteApp requires consumer key and secret'
-            )
-
         self._base_url = base_url
         self._request_token_url = request_token_url
         self._access_token_url = access_token_url
@@ -243,6 +241,7 @@ class OAuthRemoteApp(object):
         self._consumer_key = consumer_key
         self._consumer_secret = consumer_secret
         self._request_token_params = request_token_params
+        self._request_token_method = request_token_method
         self._access_token_params = access_token_params
         self._access_token_method = access_token_method
         self._content_type = content_type
@@ -251,6 +250,26 @@ class OAuthRemoteApp(object):
         self.app_key = app_key
         self.encoding = encoding
 
+        # Check for required authentication information.
+        # Skip this check if app_key is specified, since the information is
+        # specified in the Flask config, instead.
+        if not app_key:
+            req_params = self.request_token_params or {}
+            if req_params.get("signature_method") == oauthlib.oauth1.SIGNATURE_RSA:
+                # check for consumer_key and rsa_key
+                rsa_key = req_params.get("rsa_key")
+                if not consumer_key or not rsa_key:
+                    raise TypeError(
+                        "OAuthRemoteApp with RSA authentication requires "
+                        "consumer key and rsa key"
+                    )
+            else:
+                # check for consumer_key and consumer_secret
+                if not consumer_key or not consumer_secret:
+                    raise TypeError(
+                        "OAuthRemoteApp requires consumer key and secret"
+                    )
+
     @cached_property
     def base_url(self):
         return self._get_property('base_url')
@@ -279,6 +298,10 @@ class OAuthRemoteApp(object):
     def request_token_params(self):
         return self._get_property('request_token_params', {})
 
+    @cached_property
+    def request_token_method(self):
+        return self._get_property('request_token_method', 'GET')
+
     @cached_property
     def access_token_params(self):
         return self._get_property('access_token_params', {})
@@ -315,20 +338,17 @@ class OAuthRemoteApp(object):
     def make_client(self, token=None):
         # request_token_url is for oauth1
         if self.request_token_url:
+            params = self.request_token_params or {}
+            if token and isinstance(token, (tuple, list)):
+                params["resource_owner_key"] = token[0]
+                params["resource_owner_secret"] = token[1]
+
             client = oauthlib.oauth1.Client(
-                self.consumer_key, self.consumer_secret
+                client_key=self.consumer_key,
+                client_secret=self.consumer_secret,
+                **params
             )
 
-            params = self.request_token_params or {}
-            if 'signature_method' in params:
-                client.signature_method = _encode(params['signature_method'])
-            if 'rsa_key' in params:
-                client.rsa_key = _encode(params['rsa_key'])
-            if 'signature_type' in params:
-                client.signature_type = _encode(params['signature_type'])
-
-            if token and isinstance(token, (tuple, list)):
-                client.resource_owner_key, client.resource_owner_secret = token
         else:
             if token and isinstance(token, (tuple, list)):
                 token = {'access_token': token[0]}
@@ -526,21 +546,29 @@ class OAuthRemoteApp(object):
         if not realm and realms:
             realm = ' '.join(realms)
         uri, headers, _ = client.sign(
-            self.expand_url(self.request_token_url), realm=realm
+            self.expand_url(self.request_token_url),
+            http_method=self.request_token_method,
+            realm=realm,
         )
         log.debug('Generate request token header %r', headers)
-        resp, content = self.http_request(uri, headers)
-        if resp.code not in (200, 201):
-            raise OAuthException(
-                'Failed to generate request token',
-                type='token_generation_failed'
-            )
+        resp, content = self.http_request(
+            uri, headers, method=self.request_token_method,
+        )
         data = parse_response(resp, content)
-        if data is None:
+        if not data:
             raise OAuthException(
                 'Invalid token response from %s' % self.name,
                 type='token_generation_failed'
             )
+        if resp.code not in (200, 201):
+            message = 'Failed to generate request token'
+            if 'oauth_problem' in data:
+                message += ' (%s)' % data['oauth_problem']
+            raise OAuthException(
+                message,
+                type='token_generation_failed',
+                data=data,
+            )
         tup = (data['oauth_token'], data['oauth_token_secret'])
         session['%s_oauthtok' % self.name] = tup
         return tup
@@ -650,12 +678,12 @@ class OAuthRemoteApp(object):
         """Handles an OAuth callback.
 
         .. versionchanged:: 0.7
-           @authorized_handler is deprecated in favor for authorized_response.
+           @authorized_handler is deprecated in favor of authorized_response.
         """
         @wraps(f)
         def decorated(*args, **kwargs):
             log.warn(
-                '@authorized_handler is deprecated in favor for '
+                '@authorized_handler is deprecated in favor of '
                 'authorized_response'
             )
             data = self.authorized_response()
