@@ -131,11 +131,18 @@ class OAuth2Provider(object):
         if token_generator and not callable(token_generator):
             token_generator = import_string(token_generator)
 
+        refresh_token_generator = self.app.config.get(
+            'OAUTH2_PROVIDER_REFRESH_TOKEN_GENERATOR', None
+        )
+        if refresh_token_generator and not callable(refresh_token_generator):
+            refresh_token_generator = import_string(refresh_token_generator)
+
         if hasattr(self, '_validator'):
             return Server(
                 self._validator,
                 token_expires_in=expires_in,
                 token_generator=token_generator,
+                refresh_token_generator=refresh_token_generator,
             )
 
         if hasattr(self, '_clientgetter') and \
@@ -161,6 +168,7 @@ class OAuth2Provider(object):
                 validator,
                 token_expires_in=expires_in,
                 token_generator=token_generator,
+                refresh_token_generator=refresh_token_generator,
             )
         raise RuntimeError('application not bound to required getters')
 
@@ -384,9 +392,13 @@ class OAuth2Provider(object):
                 except oauth2.FatalClientError as e:
                     log.debug('Fatal client error %r', e)
                     return redirect(e.in_uri(self.error_uri))
+                except oauth2.OAuth2Error as e:
+                    log.debug('OAuth2Error: %r', e)
+                    return redirect(e.in_uri(redirect_uri or self.error_uri))
                 except Exception as e:
+                    log.warn('Exception: %r', e)
                     return redirect(add_params_to_uri(
-                        self.error_uri, {'error': e.message}
+                        self.error_uri, {'error': 'unknown'}
                     ))
 
             else:
@@ -397,9 +409,13 @@ class OAuth2Provider(object):
             except oauth2.FatalClientError as e:
                 log.debug('Fatal client error %r', e)
                 return redirect(e.in_uri(self.error_uri))
+            except oauth2.OAuth2Error as e:
+                log.debug('OAuth2Error: %r', e)
+                return redirect(e.in_uri(redirect_uri or self.error_uri))
             except Exception as e:
+                log.warn('Exception: %r', e)
                 return redirect(add_params_to_uri(
-                    self.error_uri, {'error': e.message}
+                    self.error_uri, {'error': 'unknown'}
                 ))
 
             if not isinstance(rv, bool):
@@ -414,7 +430,7 @@ class OAuth2Provider(object):
         return decorated
 
     def confirm_authorization_request(self):
-        """When consumer confirm the authrozation."""
+        """When consumer confirm the authorization."""
         server = self.server
         scope = request.values.get('scope') or ''
         scopes = scope.split()
@@ -435,14 +451,34 @@ class OAuth2Provider(object):
             log.debug('Authorization successful.')
             return create_response(*ret)
         except oauth2.FatalClientError as e:
+            log.debug('Fatal client error %r', e)
             return redirect(e.in_uri(self.error_uri))
         except oauth2.OAuth2Error as e:
-            return redirect(e.in_uri(redirect_uri))
+            log.debug('OAuth2Error: %r', e)
+            return redirect(e.in_uri(redirect_uri or self.error_uri))
         except Exception as e:
+            log.warn('Exception: %r', e)
             return redirect(add_params_to_uri(
-                self.error_uri, {'error': e.message}
+                self.error_uri, {'error': 'unknown'}
             ))
 
+    def verify_request(self, scopes):
+        """Verify current request, get the oauth data.
+
+        If you can't use the ``require_oauth`` decorator, you can fetch
+        the data in your request body::
+
+            def your_handler():
+                valid, req = oauth.verify_request(['email'])
+                if valid:
+                    return jsonify(user=req.user)
+                return jsonify(status='error')
+        """
+        uri, http_method, body, headers = extract_params()
+        return self.server.verify_request(
+            uri, http_method, body, headers, scopes
+        )
+
     def token_handler(self, f):
         """Access/refresh token handler decorator.
 
@@ -512,11 +548,7 @@ class OAuth2Provider(object):
                 if hasattr(request, 'oauth') and request.oauth:
                     return f(*args, **kwargs)
 
-                server = self.server
-                uri, http_method, body, headers = extract_params()
-                valid, req = server.verify_request(
-                    uri, http_method, body, headers, scopes
-                )
+                valid, req = self.verify_request(scopes)
 
                 for func in self._after_request_funcs:
                     valid, req = func(valid, req)
@@ -563,8 +595,12 @@ class OAuth2RequestValidator(RequestValidator):
         .. _`Section 4.1.3`: http://tools.ietf.org/html/rfc6749#section-4.1.3
         .. _`Section 6`: http://tools.ietf.org/html/rfc6749#section-6
         """
+
         if request.grant_type == 'password':
-            return True
+            client = self._clientgetter(request.client_id)
+            return (not client) or client.client_type == 'confidential' or\
+                request.client_secret
+
         auth_required = ('authorization_code', 'refresh_token')
         return 'Authorization' in request.headers and\
             request.grant_type in auth_required
@@ -620,10 +656,6 @@ class OAuth2RequestValidator(RequestValidator):
             log.debug('Authenticate failed, client not found.')
             return False
 
-        if client.client_secret != request.client_secret:
-            log.debug('Authenticate client failed, secret not match.')
-            return False
-
         # attach client on request for convenience
         request.client = client
         return True
